
/*
   This program reads numbers of type double from a data file named data.dat.
   That file should contain one number per line, with no other data.
   It should contain no more than 1000 numbers.
   A new file is created that contains the same numbers as data.dat,
   but in reverse order.  The new file is named results.dat.
   
   This program is meant as a simple example of using input and
   output files.
   
   The program depends on the class TextReader.

*/


import java.io.*;  

public class ReverseFile {

   public static void main(String[] args) {

      TextReader data;         // character input stream for reading data
      PrintWriter result;      // character outputr stream for output
      
      double[] number = new double[1000];  // array to hold the numbers
                                           //    read from the input file
      int numberCt;  // number of items stored in the array
      
      try {
         data = new TextReader(new FileReader("data.dat"));
      }
      catch (FileNotFoundException e) {
         System.out.println("Can't find file data.dat!");
         return;  // end the program by returning from main()
      }

      try {
         result = new PrintWriter(new FileWriter("result.dat"));
      }
      catch (IOException e) {
         System.out.println("Can't open file result.dat!");
         System.out.println(e.toString());
         data.close();  // close the input file
         return;  // end the program
      }
      
      try {
      
          // read the data from the input file,
          numberCt = 0;
          while (!data.eof()) {  // read to end-of-file
             number[numberCt] = data.getlnDouble();
             numberCt++;
          }
       
          // then output the numbers in reverse order
          for (int i = numberCt-1; i >= 0; i--)
             result.println(number[i]);
             
          System.out.println("Done!");

       }
       catch (TextReader.Error e) {
          // some problem reading the data from the input file
          System.out.println("Input Error: " + e.getMessage());
       }
       catch (IndexOutOfBoundsException e) {
          // must have tried to put too many numbers in the array
          System.out.println("Too many numbers in data file.");
          System.out.println("Processing has been aborted.");
       }
       finally {
          // finish by closing the files, whatever else may have happened
          data.close();
          result.close();
       }
         
   }  // end of main()

} // end of class